import os
import requests
import time
from urllib.parse import urlencode
import hmac
import hashlib
import json
from itertools import count

__version__ = "v1.1"

BASE_URL = "https://bittrex.com/api/" + __version__ + "/{service}/{method}?"
NONCE_COUNTER = count(int(time.time() * 1000))


MARKET_SET = {
    'getopenorders',
    'cancel',
    'sellmarket',
    'selllimit',
    'buymarket',
    'buylimit'
}

ACCOUNT_SET = {
    'getbalances',
    'getbalance',
    'getdepositaddress',
    'withdraw',
    'getorderhistory',
    'getorder',
    'getdeposithistory',
    'getwithdrawalhistory'
}


class Bittrex:

    def __init__(self, key=None, secret=None):

        if not key:
            key = os.environ['BITTREX_KEY']

        if not secret:
            secret = os.environ['BITTREX_SECRET']

        self.key = key
        self.secret = secret

    def query_engine(self, method, options=None):
        """
            engine of pi-bittrex
        """
        service = 'public'
        if method in MARKET_SET:
            service = 'market'
        elif method in ACCOUNT_SET:
            service = 'account'

        url = BASE_URL.format(service=service, method=method)

        if service != 'public':
            url += 'apikey={}&nonce={}&'.format(self.key, next(NONCE_COUNTER))

        if not options:
            options = {}

        url += urlencode(options)

        apisign = hmac.new(self.secret.encode(),
                           url.encode(),
                           hashlib.sha512).hexdigest()

        headers = {
            "apisign": apisign
        }

        print(url)
        return requests.get(url, headers=headers).json()

    # public
    def get_markets(self):
        """ Used to get the open and available trading markets at Bittrex
        along with other meta data.

        Endpoint:
            /public/getmarkets

        Returns:
            JSON
        """
        return self.query_engine('getmarkets')

    def get_currencies(self):
        """Used to get all supported currencies at Bittrex along with other meta data.

        Endpoint:
            /public/getcurrencies

        Returns:
            JSON
        """
        return self.query_engine('getcurrencies')

    def get_ticker(self, market):
        """Used to get the current tick values for a market.

        Endpoint:
            /public/getticker

        Args:
            market (string): string literal for the market (ex: BTC-LTC)

        Returns:
            JSON
        """
        return self.query_engine('getticker', options={"market": market})

    def get_market_summaries(self):
        """Used to get the last 24 hour summary of all active exchanges

        Endpoint:
            /public/getmarketsummaries

        Returns:
            JSON
        """
        return self.query_engine('getmarketsummaries')

    def get_market_summary(self, market):
        """ Used to get the last 24 hour summary of all active exchanges

        Endpoint:
            /public/getmarketsummary

        Args:
            market (string): string literal for the market (ex: BTC-LTC)

        Returns:
            JSON
        """
        return self.query_engine('getmarketsummary', options={"market": market})

    def get_order_book(self, market, type):
        """Used to get retrieve the orderbook for a given market

        Endpoint:
            /public/getorderbook

        Args:
            market (string): string literal for the market (ex: BTC-LTC)
            type (string): buy, sell or both to identify the type of orderbook to return.

        Returns:
            JSON

        """
        return self.query_engine('getorderbook', options={"market": market, "type": type})

    def get_market_history(self, market):
        """Used to retrieve the latest trades that have occured for a specific market.

        Endpoint:
            /public/getmarkethistory

        Args:
            market (string): string literal for the market (ex: BTC-LTC)

        Returns:
            JSON
        """
        return self.query_engine('getmarkethistory', options={"market": market})

    # market
    def buy_limit(self, market, quantity, rate):
        """Used to place a buy order in a specific market.

        Endpoint:
            /market/buylimit

        Args:
            market (string): string literal for the market (ex: BTC-LTC)
            quantity (int): the amount to purchase
            rate (int): the rate at which to place the order

        Returns:
            JSON

        """

        return self.query_engine('buylimit', options={
            "market": market,
            "quantity": quantity,
            "rate": rate,
        })

    def sell_limit(self, market, quantity, rate):
        """Used to place an sell order in a specific market.

        Endpoint:
            /market/selllimit

        Args:
            market (string): string literal for the market (ex: BTC-LTC)
            quantity (int): the amount to purchase
            rate (int): the rate at which to place the order

        Returns:
            JSON
        """
        return self.query_engine('selllimit', options={
            "market": market,
            "quantity": quantity,
            "rate": rate,
        })

    def cancel(self, uuid):
        """ Used to cancel a buy or sell order.

        Endpoint:
            /market/cancel

        Args:
            uuid (string): uuid of buy or sell order

        Returns:
            JSON
        """

        return self.query_engine('cancel', options={
            "uuid": uuid
        })

    def get_open_orders(self, market=None):
        """Get all orders that you currently have opened. A specific market can be requested

        Endpoint:
            /market/getopenorders

        Args:
            market string literal for the market (ie. BTC-LTC)

        Returns:
            JSON
        """

        return self.query_engine('getopenorders',
                                 {"market": market}if market else None)

    def get_balances(self):
        """ Used to retrieve all balances from your account

        Endpoint:
            /account/getbalances

        Returns:
            JSON

        """

        return self.query_engine('getbalances')

    def get_balance(self, currency):
        """ Used to retrieve the balance from your account for a specific currency.

        Endpoint:
            /account/getbalance

        Args:
            currency (string): string literal for the currency (ie. BTC)

        Returns:
            JSON

        """

        return self.query_engine('getbalance', options={
            "currency": currency
        })

    def get_deposit_address(self, currency):
        """ Used to retrieve or generate an address for a specific currency.
        If one does not exist, the call will fail and return ADDRESS_GENERATING
        until one is available.

        Endpoint:
            /account/getdepositaddress

        Args:
            currency (string): string literal for the currency (ie. BTC)

        Returns:
            JSON
        """
        return self.query_engine('getdepositaddress', options={
            "currency": currency
        })

    def withdraw(self, currency, quantity, address):
        """ Used to withdraw funds from your account. note: please account for txfee.

        Endpoint:
            /account/withdraw

        Args:
            currency (string): a string literal for the currency (ie. BTC)
            quantity (int): the quantity of coins to withdraw
            address (string): the address where to send the funds.

        Returns:
            JSON

        """
        return self.query_engine('withdraw', options={
            "currency": currency,
            "quantity": quantity,
            "address": address
        })

    def get_order(self, uuid):
        """ Used to retrieve a single order by uuid.

        Endpoint:
            /account/getorder

        Args:
            uuid (string): uuid of the buy or sell order

        Returns:
            JSON

        """
        return self.query_engine('withdraw', options={
            "uuid": uuid
        })

    def get_order_history(self, market=None):
        """ Used to retrieve your order history.

        Endpoint:
            /account/getorderhistory

        Args:
            market (string) [optional]: a string literal for the market 
            (ie. BTC-LTC). If ommited, will return for all markets

        Returns:
            JSON
        """
        return self.query_engine('getorderhistory',
                                 {"market": market} if market else None)

    def get_withdrawal_history(self, currency=None):
        """ Used to retrieve your withdrawal history. 

        Endpoint:
            /account/getwithdrawalhistory 

        Args:
            currency (string) [optional]:optional a string literal for 
            the currecy (ie. BTC). If omitted, will return for all currencies 

        Returns:
            JSON
        """

        return self.query_engine('getwithdrawalhistory',
                                 {"currency": currency} if currency else None)

    def get_deposit_history(self, currency=None):
        """ Used to retrieve your deposit history.

        Endpoint:
            /account/getdeposithistory 

        Args:
            currency (string) [optional]:optional a string literal for 
            the currecy (ie. BTC). If omitted, will return for all currencies 

        Returns:
            JSON
        """

        return self.query_engine('getdeposithistory',
                                 {"currency": currency} if currency else None)




from setuptools import setup


setup(
    name="p3_bittrex",
    packages=['p3_bittrex'],
    version="0.1.1",
    description="Bittrex API pacakge",
    author="Andy Hsieh",
    author_email="andy.hsieh@hotmail.com",
    license='LICENSE.txt',
    url='https://github.com/bealox/p3-bittrex',
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Environment :: Other Environment",
        "Intended Audience :: Developers",
        'License :: OSI Approved :: MIT License',
        "Operating System :: OS Independent",
        'Topic :: Office/Business :: Financial',
    ],
    install_requires=[
        "requests >= 2.18.3",
        'setuptools'
    ]

)



"""
    Requires Bittrex API to be up
    Requires setting env variables for client and secret key
"""

import unittest
import json
import os
from p3_bittrex.p3_bittrex import Bittrex


def test_basic_response(unit_test, result, method_name):
    unit_test.assertTrue(result['success'], "{0:s} failed".format(method_name))
    unit_test.assertTrue(result['message'] is not None,
                         "message not present in response")
    unit_test.assertTrue(result['result'] is not None,
                         "result not present in response")


def test_failed_response(unit_test, result, test_type):
    unit_test.assertFalse(result['success'], "{0:s} failed".format(test_type))
    unit_test.assertTrue(any(x in result['message'].lower() for x in ['exist', 'invalid']),
                         "{0:s} failed response message".format(test_type))
    unit_test.assertIsNone(result['result'],
                           "failed result in response")


def test_response_structure(unit_test, result, method_name):
    unit_test.assertTrue('success' in result, 'Invalid structure')
    unit_test.assertTrue('message' in result, 'Invalid structure')
    unit_test.assertTrue('result' in result, 'Invalid structure')


class TestBittrexPublicAPI(unittest.TestCase):
    """
        Integration tests for Bittrex Public API
    """

    def setUp(self):
        self.bittrex = Bittrex()

    def test_get_markets(self):
        actual = self.bittrex.get_markets()
        test_basic_response(self, actual, 'get_markets')

    def test_get_currencies(self):
        actual = self.bittrex.get_currencies()
        test_basic_response(self, actual, 'get_currencies')

    def test_get_ticker(self):
        self.assertRaises(TypeError, self.bittrex.get_ticker)
        actual = self.bittrex.get_ticker('BTC-LTC')
        test_basic_response(self, actual, 'get_ticker')
        false_actual = self.bittrex.get_ticker('BTC')
        test_failed_response(self, false_actual, 'get_ticker')

    def test_get_market_summaries(self):
        actual = self.bittrex.get_market_summaries()
        test_basic_response(self, actual, 'get_market_summaries')

    def test_get_market_summary(self):
        self.assertRaises(TypeError, self.bittrex.get_market_summary)
        actual = self.bittrex.get_ticker('BTC-LTC')
        test_basic_response(self, actual, 'get_market_summar')
        false_actual = self.bittrex.get_ticker('BTC')
        test_failed_response(self, false_actual, 'get_market_summar')

    def test_get_order_book(self):
        self.assertRaises(TypeError, self.bittrex.get_order_book)
        self.assertRaises(TypeError, self.bittrex.get_order_book, 'BTC-LTC')
        actual = self.bittrex.get_order_book('BTC-LTC', 'buy')
        test_basic_response(self, actual, 'get_order_book')

    def test_get_market_history(self):
        self.assertRaises(TypeError, self.bittrex.get_market_history)
        actual = self.bittrex.get_market_history('BTC-LTC')
        test_basic_response(self, actual, 'get_market_history')


class TestBittrexMarketAPI(unittest.TestCase):

    def setUp(self):
        self.bittrex = Bittrex()

    def test_secret_and_key(self):
        try:
            os.environ['BITTREX_KEY']
            os.environ['BITTREX_SECRET']
        except KeyError:
            self.fail("Requires BITTREX_KEY and BITTREX_SECRET env variables")

    def test_buy_limit(self):
        self.assertRaises(TypeError, self.bittrex.buy_limit)
        actual = self.bittrex.buy_limit('BTC-LTC', 1, 1)
        test_response_structure(self, actual, 'buy_limit')

    def test_sell_limit(self):
        self.assertRaises(TypeError, self.bittrex.buy_limit)
        actual = self.bittrex.sell_limit('BTC-LTC', 1, 1)
        test_response_structure(self, actual, 'sell_limit')

    def test_cancel(self):
        self.assertRaises(TypeError, self.bittrex.cancel)
        # provide invalid uuid but test the json structure
        actual = self.bittrex.cancel('BTC-LTC')
        test_response_structure(self, actual, 'cancel')

    def test_open_orders(self):
        invalid_actual = self.bittrex.get_open_orders('Invalid Market')
        no_param_actual = self.bittrex.get_open_orders()
        actual = self.bittrex.get_open_orders('BTC-LTC')
        test_failed_response(self, invalid_actual, 'get_open_orders')
        test_basic_response(self, no_param_actual, 'get_open_orders')
        test_basic_response(self, actual, 'get_open_orders')


class TestBittrexAccountAPI(unittest.TestCase):

    def setUp(self):
        self.bittrex = Bittrex()

    def test_get_balances(self):
        actual = self.bittrex.get_balances()
        test_basic_response(self, actual, 'get_balances')

    def test_get_balance(self):
        self.assertRaises(TypeError, self.bittrex.get_balance)
        actual = self.bittrex.get_balance('BTC')
        test_basic_response(self, actual, 'get_balance')
        invalid_actual = self.bittrex.get_balance('Invalid currency')
        test_failed_response(self, invalid_actual, 'get_balance')

    def test_get_deposit_address(self):
        self.assertRaises(TypeError, self.bittrex.get_deposit_address)
        actual = self.bittrex.get_deposit_address('BTC')
        test_basic_response(self, actual, 'get_deposit_address')
        invalid_actual = self.bittrex.get_deposit_address('Invalid currency')
        test_failed_response(self, invalid_actual, 'get_deposit_address')

    def test_withdraw(self):
        self.assertRaises(TypeError, self.bittrex.withdraw)

    def test_get_order(self):
        self.assertRaises(TypeError, self.bittrex.get_order)
        actual = self.bittrex.get_order('test')
        test_response_structure(self, actual, 'get_order')

    def test_get_order_history(self):
        actual = self.bittrex.get_order_history()
        test_basic_response(self, actual, 'get_order_history')
        actual = self.bittrex.get_order_history('BTC-LTC')
        test_basic_response(self, actual, 'get_order_history')

    def test_get_withdrawal_historyself(self):
        actual = self.bittrex.get_withdrawal_history()
        test_basic_response(self, actual, 'get_withdrawal_history')
        actual = self.bittrex.get_withdrawal_history('BTC')
        test_basic_response(self, actual, 'get_withdrawal_history')

    def test_get_deposit_history(self):
        actual = self.bittrex.get_deposit_history()
        test_basic_response(self, actual, 'get_deposit_history')
        actual = self.bittrex.get_deposit_history('BTC')
        test_basic_response(self, actual, 'get_deposit_history')


if __name__ == '__main__':
    unittest.main()
